﻿// $Id: UlozToDownload.cs 19 2008-03-24 19:29:57Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.IO;
using System.Text.RegularExpressions;
using OSD.PluginBase;

namespace OSD.Plugins.UlozTo
{
    internal class UlozToDownload : Download
    {
        //private const int CONNECTION_TIMEOUT = 60000;
        private readonly int waitTime = 0;
        private long position = 0;
        private long size = 0;

        private DownloadStatus status = DownloadStatus.Connecting;
        private string statusText = "";

        ///<summary>
        /// Konstruktor stahovaci tridy
        ///</summary>
        ///<param name="url">url souboru, ktery se bude stahovat</param>
        ///<param name="parrent">trida DownloadPluginu, kteremu nalezi tato trida</param>
        public UlozToDownload(string url, PluginBase.DownloadPlugin parrent) : base(url, parrent) {}

        ///<summary>
        /// Vraci stav stahovani
        ///</summary>
        public override DownloadStatus Status
        {
            get { return status; }
        }

        ///<summary>
        /// Vraci stav stahovani v textove podobe. Muze take obsahovat dalsi texty, ktery vratil server, odkud se stahuje. 
        ///</summary>
        public override string StatusText
        {
            get { return statusText; }
        }

        ///<summary>
        /// Vraci velikost stahovaneho souboru. Pokud jeste samotny proces stahovani nezacal vraci 0.
        ///</summary>
        public override long Size
        {
            get { return size; }
        }

        ///<summary>
        /// Vraci pocet bajtu, ktere jiz byli stahnuty
        ///</summary>
        public override long Position
        {
            get { return position; }
        }

        ///<summary>
        /// Vraci doporuceny cas, ktery je nutno pockat pro dalsi akci. Nastaveni tohoto gettru se provede pri Status == DownloadStatus.Waiting a Status == DownloadStatus.Timeout.
        ///</summary>
        public override int WaitTime
        {
            get { return waitTime; }
        }

        ///<summary>
        /// Funkce starajici se smazani pouzitych zdroju. Zavola se automaticky po zruseni tridy.
        ///</summary>
        protected override void ClearResources() {}

        ///<summary>
        /// Hlavni metoda starijici se o stahovani. Metoda je spustena v samostatnem vlakne. Zdedena trida musi tuto metodu implementovat.
        ///</summary>
        protected override void DownloadThread()
        {
            string content;
            GroupCollection group;

            status = DownloadStatus.Connecting;
            statusText = "Připojuji se...";
            OnStatusChanged(new DownloadStatusChangedEventArgs(status, statusText));

            try
            {
                wd.Clear();
                wd.Url = Url;
                wd.Method = MethodType.Get;
                wd.Send();
                content = wd.ResponseText;
            }
            catch (Exception ex)
            {
                status = DownloadStatus.Timeout;
                statusText = "Připojení se nezdařilo.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, ex.Message));
                return;
            }

            if (content.Length == 0)
            {
                status = DownloadStatus.Timeout;
                statusText = "Odkaz nenalezen.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, statusText));
                return;
            }

            if (
                !Util.RegexMatch("<a href=\"(?<url>http://dlg.uloz.to/[^/]+/\\d+/(?<filename>[^\"]+))\">", content,
                                 out group))
            {
                //TODO otestovat, zda je soubor smazan
                status = DownloadStatus.Timeout;
                statusText = "Odkaz nenalezen.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, statusText));
                return;
            }

            string downloadUrl = group["url"].Value;

            status = DownloadStatus.Downloading;
            statusText = "Stahuji...";
            OnStatusChanged(new DownloadStatusChangedEventArgs(status, statusText));

            string localFilename =
                Path.Combine(Parrent.Settings.getString("osd/settings/destination_path", @"c:\"),
                             group["filename"].Value);
            try
            {
                wd.Clear();
                wd.Url = downloadUrl;
                wd.Method = MethodType.Get;
                //pokud existuje, pokusime se nastavit, ze chceme pokracovat
                if (File.Exists(localFilename))
                    wd.AddRange = new FileInfo(localFilename).Length;

                wd.Send();
                wd.SaveToFile(localFilename, delegate(object sender, SaveToFileProgressChangedEventArgs e)
                                                 {
                                                     position = e.BytesReceived;
                                                     size = e.TotalBytesToReceive;
                                                     OnPositionChanged(new PositionChangedEventArgs(position, size));
                                                 });
            }
            catch (Exception e)
            {
                status = DownloadStatus.IOError;
                statusText = "Stahování se nezdařilo.";
                OnDownloadFailed(new DownloadFailedEventArgs(status, statusText, e.Message));
                return;
            }
            status = DownloadStatus.Downloaded;
            statusText = "";

            OnStatusChanged(new DownloadStatusChangedEventArgs(status, statusText));
            OnDownloadComplete(new DownloadCompleteEventArgs(Url, localFilename));
        }
    }
}